/*
	MitPit

	   Mitsubishi-Pioneer.

	igorkov / 2016 / fsp@igorkov.org
 */
#include <avr/io.h>
#include <avr/interrupt.h>

// ADC0 - input!
//   :
// Vol- 3.12 kOm, 
// Vol+ 2.12 kOm, 
// DOWN 1.29 kOm, 
// UP   0.74 kOm, 
// MODE 0.26 kOm.
//   : 71.1 kOm. 
//    1. :
//  3120 / (3120 + 1000) * 1024 = 775
//  2120 / (2120 + 1000) * 1024 = 696
//  1290 / (1290 + 1000) * 1024 = 577
//   740 / ( 740 + 1000) * 1024 = 433
//   260 / ( 260 + 1000) * 1024 = 211
// 71100 / (71100+ 1000) * 1024 = 1010
//    1.5. :
//  3120 / (3120 + 1500) * 1024 = 692
//  2120 / (2120 + 1500) * 1024 = 600
//  1290 / (1290 + 1500) * 1024 = 473
//   740 / ( 740 + 1500) * 1024 = 338
//   260 / ( 260 + 1500) * 1024 = 151
// 71100 / (71100+ 1500) * 1024 = 1002

#if 1 // R=1k
#define ADC_RANGE 30
#define MODE_MIN_LEVEL     (211-ADC_RANGE)
#define MODE_MAX_LEVEL     (211+ADC_RANGE)
#define UP_MIN_LEVEL       (433-ADC_RANGE)
#define UP_MAX_LEVEL       (433+ADC_RANGE)
#define DOWN_MIN_LEVEL     (577-ADC_RANGE)
#define DOWN_MAX_LEVEL     (577+ADC_RANGE)
#define VOLUP_MIN_LEVEL    (696-ADC_RANGE)
#define VOLUP_MAX_LEVEL    (696+ADC_RANGE)
#define VOLDOWN_MIN_LEVEL  (775-ADC_RANGE)
#define VOLDOWN_MAX_LEVEL  (775+ADC_RANGE)
#else // R=1.5k
#define ADC_RANGE 40
#define MODE_MIN_LEVEL     (151-ADC_RANGE)
#define MODE_MAX_LEVEL     (151+ADC_RANGE)
#define UP_MIN_LEVEL       (338-ADC_RANGE)
#define UP_MAX_LEVEL       (338+ADC_RANGE)
#define DOWN_MIN_LEVEL     (473-ADC_RANGE)
#define DOWN_MAX_LEVEL     (473+ADC_RANGE)
#define VOLUP_MIN_LEVEL    (600-ADC_RANGE)
#define VOLUP_MAX_LEVEL    (600+ADC_RANGE)
#define VOLDOWN_MIN_LEVEL  (692-ADC_RANGE)
#define VOLDOWN_MAX_LEVEL  (692+ADC_RANGE)
#endif

// mode  up    down      vol+      vol-
// PA1,  PA2,  PA3+PA5,  PA4+PA5,  PA5

void set_nop(void)
{
	DDRA  &= ~0xFE;
	PORTA &= ~0xFE;
}

inline void set_mode(void)
{
	DDRA  |=  0x02;
	PORTA &= ~0x02;
}

inline void set_up(void)
{
	DDRA  |=  0x04;
	PORTA &= ~0x04;
}

inline void set_down(void)
{
	DDRA  |=  (0x08|0x20);
	PORTA &= ~(0x08|0x20);
}

inline void set_volup(void)
{
	DDRA  |=  (0x10|0x20);
	PORTA &= ~(0x10|0x20);
}

inline void set_voldown(void)
{
	DDRA  |=  0x20;
	PORTA &= ~0x20;
}

inline uint16_t adc_get(void)
{
	ADCSRA |= 0x40; // start conversion
	while ((ADCSRA & 0x10)==0);
	ADCSRA |= 0x10;
	return ADC;
}

int main(void)
{
	uint16_t value;
	uint8_t premode = -1;

	DDRA    = 0;
	PORTA   = 0;

	// init pio:
	set_nop();
	// init adc:
	DIDR0   = 0x01; // ADC0 digital input disable
	ACSR    = 0x80; // Comparator disable
	ADCSRA  = (1<<ADEN)|(1<<ADPS2)|(0<<ADPS1)|(0<<ADPS0); // ADC enable, Divider = 16	
	ADMUX   = (0<<REFS1)|(0<<REFS0)| // VCC as Vref
				(0<<MUX0)|(0<<MUX1)|(0<<MUX2)|(0<<MUX3)|(0<<MUX4); // ADC0 as input
	// no interrupts:
	cli();

	while (1)
	{
		value = adc_get();
		if (value <= MODE_MAX_LEVEL &&
			value >= MODE_MIN_LEVEL)
		{
			if (premode != 1) {
				set_nop();
				set_mode();
			}
			premode = 1;
		} else
		if (value <= UP_MAX_LEVEL &&
			value >= UP_MIN_LEVEL)
		{
			if (premode != 2) {
				set_nop();
				set_up();
			}
			premode = 2;
		} else
		if (value <= DOWN_MAX_LEVEL &&
			value >= DOWN_MIN_LEVEL)
		{
			if (premode != 3) {
				set_nop();
				set_down();
			}
			premode = 3;
		} else
		if (value <= VOLUP_MAX_LEVEL &&
			value >= VOLUP_MIN_LEVEL)
		{
			if (premode != 4) {
				set_nop();
				set_volup();
			}
			premode = 4;
		} else
		if (value <= VOLDOWN_MAX_LEVEL &&
			value >= VOLDOWN_MIN_LEVEL)
		{
			if (premode != 5) {
				set_nop();
				set_voldown();
			}
			premode = 5;
		} else {
			set_nop();
			premode = -1;
		}
	}
	return 0;
}
